#######################################################################
# cumulative cross-section script - author Kai Urban - last edit 25.02.2022
#######################################################################

# load necessary packages
from scipy import integrate

from DMgamSpec.zerobins import ZBwino, ZBhig
from DMgamSpec.diffcross import diffwino, diffhig
from DMgamSpec.cumulcross import zerobinsize

#######################################################################
# Bin cross-section in energy
# (output dimension 10^(-26) cm^3/s)
# function arguments - mass [TeV], E1 [TeV], E2 [TeV], model (string), SF
# (string), ZBsize (string - optional)
#######################################################################

def binnedwino(mchi, E1, E2, SF, ZBsize, rel):
    # determine zero-bin size
    ZBvalue = zerobinsize(ZBsize)

    # add points for integrator, to achieve more precise results
    # locate add 1/3 and 2/3 between E1/mchi and E2/mchi
    ptlist = [(2/3 * E1 + 1/3 * E2) / mchi , (1/3 * E1 + 2/3 * E2) / mchi]

    # if bin inside the zeroth bin, add zero-bin on top, otherwise integrate
    # from bin limits, zero-bin is checked in calling function
    # binnedcrosssection function
    if (mchi - E2) < ZBvalue * mchi:
        return ZBwino(mchi, SF, ZBsize) + integrate.quad(diffwino, E1/mchi, \
                1-ZBvalue, limit=250, epsrel=10**(rel), epsabs = 0, \
                points = ptlist, args=(mchi,SF,))[0]
    else:
        return integrate.quad(diffwino, E1/mchi, E2/mchi, limit=250, \
                epsrel=10**(rel), epsabs = 0, points = ptlist, \
                args=(mchi,SF,))[0]

def binnedhig(mchi, E1, E2, SF, ZBsize, rel):
    # determine zero-bin size
    ZBvalue = zerobinsize(ZBsize)

    # add points for integrator, to achieve more precise results
    # locate add 1/3 and 2/3 between E1/mchi and E2/mchi
    ptlist = [(2/3 * E1 + 1/3 * E2) / mchi , (1/3 * E1 + 2/3 * E2) / mchi]

    # if bin inside the zeroth bin, add zero-bin on top, otherwise integrate
    # from bin limits
    if (mchi - E2) < ZBvalue * mchi:
        return ZBhig(mchi, SF, ZBsize) + integrate.quad(diffhig, E1/mchi, \
                1-ZBvalue, limit=250, epsrel=10**(rel), epsabs = 0, \
                points = ptlist, args=(mchi,SF,))[0]
    else:
        return integrate.quad(diffhig, E1/mchi, E2/mchi, limit=250, \
                epsrel=10**(rel), epsabs = 0, points = ptlist, \
                args=(mchi,SF,))[0]
