#######################################################################
# load data script - author Kai Urban - last edit 04.02.2022
#######################################################################

import numpy as np
from scipy import interpolate 

#######################################################################
# Load necessary Sommerfeld tables and interpolate them - wino model
#######################################################################

SF_path = './data/sommerfeld/' # path to sommerfeld files

SFWinoLOvmin3Tab = np.genfromtxt(SF_path + 'wino/WinoSIJ_v-3_LO_dm164.csv', delimiter=',')
SFWinoNLOvmin3Tab = np.genfromtxt(SF_path + 'wino/WinoSIJ_v-3_NLO_dm164.csv', delimiter=',')

SFWinoLOvmin4Tab = np.genfromtxt(SF_path + 'wino/WinoSIJ_v-4_LO_dm164.csv', delimiter=',')
SFWinoNLOvmin4Tab = np.genfromtxt(SF_path + 'wino/WinoSIJ_v-4_NLO_dm164.csv', delimiter=',')

SFWinoLOvmin5Tab = np.genfromtxt(SF_path + 'wino/WinoSIJ_v-5_LO_dm164.csv', delimiter=',')
SFWinoNLOvmin5Tab = np.genfromtxt(SF_path + 'wino/WinoSIJ_v-5_NLO_dm164.csv', delimiter=',')

# interpolate above tables, order of outputs S00, Re[S0p], Im[S0p], Spm
SFLOwinomin3 = interpolate.interp1d(SFWinoLOvmin3Tab[:,0], SFWinoLOvmin3Tab[:,[1,2,3,4]].T, kind='cubic')
SFNLOwinomin3 = interpolate.interp1d(SFWinoNLOvmin3Tab[:,0], SFWinoNLOvmin3Tab[:,[1,2,3,4]].T, kind='cubic')

SFLOwinomin4 = interpolate.interp1d(SFWinoLOvmin4Tab[:,0], SFWinoLOvmin4Tab[:,[1,2,3,4]].T, kind='cubic')
SFNLOwinomin4 = interpolate.interp1d(SFWinoNLOvmin4Tab[:,0], SFWinoNLOvmin4Tab[:,[1,2,3,4]].T, kind='cubic')

SFLOwinomin5 = interpolate.interp1d(SFWinoLOvmin5Tab[:,0], SFWinoLOvmin5Tab[:,[1,2,3,4]].T, kind='cubic')
SFNLOwinomin5 = interpolate.interp1d(SFWinoNLOvmin5Tab[:,0], SFWinoNLOvmin5Tab[:,[1,2,3,4]].T, kind='cubic')

#######################################################################
# Load necessary Sommerfeld tables and interpolate them - higgsino
#######################################################################

# load Sommerfeld tables for v = 10^(-3,-4,-5), deltam = 355 MeV, 
# deltamN = 20,0.15 MeV  and the LO and NLO potentials
SFHigLOvmin3dmN20Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-3_LO_dm355_dmN20MeV.csv', delimiter=',')
SFHigNLOvmin3dmN20Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-3_NLO_dm355_dmN20MeV.csv', delimiter=',')

SFHigLOvmin4dmN20Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-4_LO_dm355_dmN20MeV.csv', delimiter=',')
SFHigNLOvmin4dmN20Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-4_NLO_dm355_dmN20MeV.csv', delimiter=',')

SFHigLOvmin5dmN20Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-5_LO_dm355_dmN20MeV.csv', delimiter=',')
SFHigNLOvmin5dmN20Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-5_NLO_dm355_dmN20MeV.csv', delimiter=',')

SFHigLOvmin3dmN015Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-3_LO_dm355_dmN150keV.csv', delimiter=',')
SFHigNLOvmin3dmN015Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-3_NLO_dm355_dmN150keV.csv', delimiter=',')

SFHigLOvmin4dmN015Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-4_LO_dm355_dmN150keV.csv', delimiter=',')
SFHigNLOvmin4dmN015Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-4_NLO_dm355_dmN150keV.csv', delimiter=',')

SFHigLOvmin5dmN015Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-5_LO_dm355_dmN150keV.csv', delimiter=',')
SFHigNLOvmin5dmN015Tab = np.genfromtxt(SF_path + 'higgsino/HigSIJ_v-5_NLO_dm355_dmN150keV.csv', delimiter=',')

# interpolate above tables, order of outputs
# S_1111, Re[S_1122], Im[S_1122], Re[S_11+-], Im[S_11+-], S_2222, Re[S_22+-], Im[S_22+-], S_+-+-
SFLOHigmin3dmN20 = interpolate.interp1d(SFHigLOvmin3dmN20Tab[:,0], SFHigLOvmin3dmN20Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')
SFNLOHigmin3dmN20 = interpolate.interp1d(SFHigNLOvmin3dmN20Tab[:,0], SFHigNLOvmin3dmN20Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')

SFLOHigmin4dmN20 = interpolate.interp1d(SFHigLOvmin4dmN20Tab[:,0], SFHigLOvmin4dmN20Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')
SFNLOHigmin4dmN20 = interpolate.interp1d(SFHigNLOvmin4dmN20Tab[:,0], SFHigNLOvmin4dmN20Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')

SFLOHigmin5dmN20 = interpolate.interp1d(SFHigLOvmin5dmN20Tab[:,0], SFHigLOvmin5dmN20Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')
SFNLOHigmin5dmN20 = interpolate.interp1d(SFHigNLOvmin5dmN20Tab[:,0], SFHigNLOvmin5dmN20Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')

SFLOHigmin3dmN015 = interpolate.interp1d(SFHigLOvmin3dmN015Tab[:,0], SFHigLOvmin3dmN015Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')
SFNLOHigmin3dmN015 = interpolate.interp1d(SFHigNLOvmin3dmN015Tab[:,0], SFHigNLOvmin3dmN015Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')

SFLOHigmin4dmN015 = interpolate.interp1d(SFHigLOvmin4dmN015Tab[:,0], SFHigLOvmin4dmN015Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')
SFNLOHigmin4dmN015 = interpolate.interp1d(SFHigNLOvmin4dmN015Tab[:,0], SFHigNLOvmin4dmN015Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')

SFLOHigmin5dmN015 = interpolate.interp1d(SFHigLOvmin5dmN015Tab[:,0], SFHigLOvmin5dmN015Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')
SFNLOHigmin5dmN015 = interpolate.interp1d(SFHigNLOvmin5dmN015Tab[:,0], SFHigNLOvmin5dmN015Tab[:,[1,2,3,4,5,6,7,8,9]].T, kind='cubic')

#######################################################################
# pick appropriate Sommerfeld table, and use interpolating function at given
# mass
#######################################################################
def winoSFactor(mchi, SF):
    Sfactor = np.zeros(3)
    if SF == 'LO -3':
        Sfactor = SFLOwinomin3(mchi)
    elif SF == 'LO -4':
        Sfactor = SFLOwinomin4(mchi)
    elif SF == 'LO -5':
        Sfactor = SFLOwinomin5(mchi)
    elif SF == 'NLO -3':
        Sfactor = SFNLOwinomin3(mchi)
    elif SF == 'NLO -4':
        Sfactor = SFNLOwinomin4(mchi)
    elif SF == 'NLO -5':
        Sfactor = SFNLOwinomin5(mchi)
    else:
        print('!!! Error - no valid Sommerfeld factor argument provided !!!')
    return Sfactor

def higgsinoSFactor(mchi, SF):
    Sfactor = np.zeros(3)
    if SF == 'LO -3 dm 355 dmN 20':
        Sfactor = SFLOHigmin3dmN20(mchi)
    elif SF == 'LO -4 dm 355 dmN 20':
        Sfactor = SFLOHigmin4dmN20(mchi)
    elif SF == 'LO -5 dm 355 dmN 20':
        Sfactor = SFLOHigmin5dmN20(mchi)
    elif SF == 'LO -3 dm 355 dmN 0.15':
        Sfactor = SFLOHigmin3dmN20(mchi)
    elif SF == 'LO -4 dm 355 dmN 0.15':
        Sfactor = SFLOHigmin4dmN20(mchi)
    elif SF == 'LO -5 dm 355 dmN 015':
        Sfactor = SFLOHigmin5dmN20(mchi)
    elif SF == 'NLO -3 dm 355 dmN 20':
        Sfactor = SFNLOHigmin3dmN20(mchi)
    elif SF == 'NLO -4 dm 355 dmN 20':
        Sfactor = SFNLOHigmin4dmN20(mchi)
    elif SF == 'NLO -5 dm 355 dmN 20':
        Sfactor = SFNLOHigmin5dmN20(mchi)
    elif SF == 'NLO -3 dm 355 dmN 015':
        Sfactor = SFNLOHigmin3dmN20(mchi)
    elif SF == 'NLO -4 dm 355 dmN 015':
        Sfactor = SFNLOHigmin4dmN20(mchi)
    elif SF == 'NLO -5 dm 355 dmN 015':
        Sfactor = SFNLOHigmin5dmN20(mchi)
    else:
        print('!!! Error - no valid Sommerfeld factor argument provided !!!')
    return Sfactor
